package ru.dbotthepony.kstarbound.player

import com.google.common.collect.ImmutableList
import ru.dbotthepony.kstarbound.defs.player.InventoryConfig
import ru.dbotthepony.kstarbound.util.ItemDescriptor
import java.util.function.Predicate

class AvatarBag(val avatar: Avatar, val config: InventoryConfig.Bag, val filter: Predicate<ItemDescriptor>) {
	private val slots = ImmutableList.builder<Slot>().let {
		for (i in 0 until config.size) {
			it.add(Slot())
		}

		it.build()
	}

	private class Slot {
		var item: ItemDescriptor? = null

		fun mergeFrom(value: ItemDescriptor, simulate: Boolean) {
			if (item == null) {
				if (!simulate) {
					item = value.copy().also { it.count = value.count.coerceAtMost(value.item!!.value.maxStack) }
				}

				value.count -= value.item!!.value.maxStack
			} else {
				item!!.mergeFrom(value, simulate)
			}
		}
	}

	operator fun set(index: Int, value: ItemDescriptor?) {
		slots[index].item = value
	}

	operator fun get(index: Int): ItemDescriptor? {
		return slots[index].item
	}

	fun put(item: ItemDescriptor, simulate: Boolean): ItemDescriptor {
		if (!filter.test(item))
			return item

		val copy = item.copy()

		for (slot in slots.indices) {
			slots[slot].mergeFrom(copy, simulate)

			if (copy.isEmpty) {
				return copy
			}
		}

		return copy
	}
}
